@@ -33,7 +33,8 @@
 #include "keep-alive.h"
 #include "gjs_gi_trace.h"
 
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <gjs/compat.h>
 
 #include <util/log.h>
 
@@ -120,26 +121,29 @@ init_g_param_from_property(JSContext  *context,
 static JSBool
 object_instance_get_prop(JSContext *context,
                          JSObject  *obj,
-                         jsval      id,
+                         jsid       id,
                          jsval     *value_p)
 {
     ObjectInstance *priv;
-    const char *name;
+    char *name;
     char *gname;
     GParamSpec *param;
     GValue gvalue = { 0, };
+    JSBool ret = JS_TRUE;
 
-    if (!gjs_get_string_id(id, &name))
+    if (!gjs_get_string_id(context, id, &name))
         return JS_TRUE; /* not resolved, but no error */
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GOBJECT,
                      "Get prop '%s' hook obj %p priv %p", name, obj, priv);
 
-    if (priv == NULL)
-        return JS_FALSE; /* wrong class passed in */
-    if (priv->gobj == NULL)
-        return JS_TRUE; /* prototype, not an instance. */
+    if (priv == NULL) {
+        ret = JS_FALSE; /* wrong class passed in */
+        goto out;
+    }
+    if (priv->gobj == NULL) /* prototype, not an instance. */
+        goto out;
 
     gname = gjs_hyphen_from_camel(name);
     param = g_object_class_find_property(G_OBJECT_GET_CLASS(priv->gobj),
@@ -148,12 +152,11 @@ object_instance_get_prop(JSContext *context,
 
     if (param == NULL) {
         /* leave value_p as it was */
-        return JS_TRUE;
+        goto out;
     }
 
-    if ((param->flags & G_PARAM_READABLE) == 0) {
-        return JS_TRUE;
-    }
+    if ((param->flags & G_PARAM_READABLE) == 0)
+        goto out;
 
     gjs_debug_jsprop(GJS_DEBUG_GOBJECT,
                      "Overriding %s with GObject prop %s",
@@ -164,11 +167,14 @@ object_instance_get_prop(JSContext *context,
                           &gvalue);
     if (!gjs_value_from_g_value(context, value_p, &gvalue)) {
         g_value_unset(&gvalue);
-        return JS_FALSE;
+        ret = JS_FALSE;
+        goto out;
     }
     g_value_unset(&gvalue);
 
-    return JS_TRUE;
+ out:
+    g_free(name);
+    return ret;
 }
 
 /* a hook on setting a property; set value_p to override property value to
@@ -177,33 +183,39 @@ object_instance_get_prop(JSContext *context,
 static JSBool
 object_instance_set_prop(JSContext *context,
                          JSObject  *obj,
-                         jsval      id,
+                         jsid       id,
+#ifdef HAVE_JS_STRICTPROPERTYSTUB
+                         JSBool     strict,
+#endif
                          jsval     *value_p)
 {
     ObjectInstance *priv;
-    const char *name;
+    char *name;
     GParameter param = { NULL, { 0, }};
+    JSBool ret = JS_TRUE;
 
-    if (!gjs_get_string_id(id, &name))
+    if (!gjs_get_string_id(context, id, &name))
         return JS_TRUE; /* not resolved, but no error */
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GOBJECT,
                      "Set prop '%s' hook obj %p priv %p", name, obj, priv);
 
-    if (priv == NULL)
-        return JS_FALSE;  /* wrong class passed in */
-    if (priv->gobj == NULL)
-        return JS_TRUE; /* prototype, not an instance. */
+    if (priv == NULL) {
+        ret = JS_FALSE;  /* wrong class passed in */
+        goto out;
+    }
+    if (priv->gobj == NULL) /* prototype, not an instance. */
+        goto out;
 
     switch (init_g_param_from_property(context, name,
                                        *value_p,
                                        G_TYPE_FROM_INSTANCE(priv->gobj),
                                        &param)) {
     case SOME_ERROR_OCCURRED:
-        return JS_FALSE;
+        ret = JS_FALSE;
     case NO_SUCH_G_PROPERTY:
-        return JS_TRUE;
+        goto out;
     case VALUE_WAS_SET:
         break;
     }
@@ -219,7 +231,9 @@ object_instance_set_prop(JSContext *context,
      * getter/setter maybe, don't know if that is better.
      */
 
-    return JS_TRUE;
+ out:
+    g_free(name);
+    return ret;
 }
 
 /*
@@ -238,28 +252,33 @@ object_instance_set_prop(JSContext *context,
 static JSBool
 object_instance_new_resolve(JSContext *context,
                             JSObject  *obj,
-                            jsval      id,
+                            jsid       id,
                             uintN      flags,
                             JSObject **objp)
 {
     ObjectInstance *priv;
-    const char *name;
+    char *name;
+    JSBool ret = JS_FALSE;
 
     *objp = NULL;
 
-    if (!gjs_get_string_id(id, &name))
+    if (!gjs_get_string_id(context, id, &name))
         return JS_TRUE; /* not resolved, but no error */
 
     priv = priv_from_js(context, obj);
 
     gjs_debug_jsprop(GJS_DEBUG_GOBJECT,
-                     "Resolve prop '%s' hook obj %p priv %p gobj %p %s",
-                     name, obj, priv, priv ? priv->gobj : NULL,
-                     (priv && priv->gobj) ?
-                     g_type_name_from_instance((GTypeInstance*) priv->gobj) : "(type unknown)");
+                     "Resolve prop '%s' hook obj %p priv %p (%s.%s) gobj %p %s",
+                     name,
+                     obj,
+                     priv,
+                     priv ? g_base_info_get_namespace (priv->info) : "",
+                     priv ? g_base_info_get_name (priv->info) : "",
+                     priv ? priv->gobj : NULL,
+                     (priv && priv->gobj) ? g_type_name_from_instance((GTypeInstance*) priv->gobj) : "(type unknown)");
 
     if (priv == NULL)
-        return JS_FALSE; /* we are the wrong class */
+        goto out; /* we are the wrong class */
 
     if (priv->gobj == NULL) {
         /* We are the prototype, so look for methods and other class properties */
@@ -268,41 +287,23 @@ object_instance_new_resolve(JSContext *context,
         /* find_method does not look at methods on parent classes,
          * we rely on javascript to walk up the __proto__ chain
          * and find those and define them in the right prototype.
+         *
+         * Note that if it isn't a method on the object, since JS
+         * lacks multiple inheritance, we're sticking the iface
+         * methods in the object prototype, which means there are many
+         * copies of the iface methods (one per object class node that
+         * introduces the iface)
          */
-        method_info = g_object_info_find_method(priv->info,
-                                                name);
 
-        /* If it isn't a method on the object, see if it's one on an
-         * iface the object implements. Note that since JS lacks
-         * multiple inheritance, we stick the iface methods in the
-         * object prototype, which means there are many copies of the
-         * iface methods (one per object class node that introduces
-         * the iface)
+        method_info = g_object_info_find_method_using_interfaces(priv->info,
+                                                                 name,
+                                                                 NULL);
+
+        /**
+         * Search through any interfaces implemented by the GType;
+         * this could be done better.  See
+         * https://bugzilla.gnome.org/show_bug.cgi?id=632922
          */
-        if (method_info == NULL) {
-            int n_interfaces;
-            int i;
-
-            n_interfaces = g_object_info_get_n_interfaces(priv->info);
-
-            for (i = 0; i < n_interfaces; ++i) {
-                GIInterfaceInfo *iface_info;
-
-                iface_info = g_object_info_get_interface(priv->info, i);
-
-                method_info = g_interface_info_find_method(iface_info, name);
-
-                g_base_info_unref( (GIBaseInfo*) iface_info);
-
-                if (method_info != NULL) {
-                    gjs_debug(GJS_DEBUG_GOBJECT,
-                              "Found method %s in interface %d implemented by object",
-                              name, i);
-                    break;
-                }
-            }
-        }
-
         if (method_info == NULL) {
             GType *interfaces;
             guint n_interfaces;
@@ -357,7 +358,8 @@ object_instance_new_resolve(JSContext *context,
                           g_base_info_get_namespace( (GIBaseInfo*) priv->info),
                           g_base_info_get_name( (GIBaseInfo*) priv->info));
                 g_base_info_unref( (GIBaseInfo*) method_info);
-                return JS_TRUE;
+                ret = JS_TRUE;
+                goto out;
             }
 
             gjs_debug(GJS_DEBUG_GOBJECT,
@@ -369,7 +371,7 @@ object_instance_new_resolve(JSContext *context,
 
             if (gjs_define_function(context, obj, method_info) == NULL) {
                 g_base_info_unref( (GIBaseInfo*) method_info);
-                return JS_FALSE;
+                goto out;
             }
 
             *objp = obj; /* we defined the prop in obj */
@@ -405,7 +407,10 @@ object_instance_new_resolve(JSContext *context,
         }
     }
 
-    return JS_TRUE;
+    ret = JS_TRUE;
+ out:
+    g_free(name);
+    return ret;
 }
 
 static void
@@ -458,49 +463,49 @@ object_instance_props_to_g_parameters(JSContext   *context,
         return JS_FALSE;
     }
 
-    prop_id = JSVAL_VOID;
+    prop_id = JSID_VOID;
     if (!JS_NextProperty(context, iter, &prop_id))
         return JS_FALSE;
 
-    if (prop_id != JSVAL_VOID) {
+    if (!JSID_IS_VOID(prop_id)) {
         gparams = g_array_new(/* nul term */ FALSE, /* clear */ TRUE,
                               sizeof(GParameter));
     } else {
         return JS_TRUE;
     }
 
-    while (prop_id != JSVAL_VOID) {
-        jsval nameval;
-        const char *name;
+    while (!JSID_IS_VOID(prop_id)) {
+        char *name;
         jsval value;
         GParameter gparam = { NULL, { 0, }};
 
-        if (!JS_IdToValue(context, prop_id, &nameval))
+        if (!gjs_get_string_id(context, prop_id, &name))
             goto free_array_and_fail;
 
-        if (!gjs_get_string_id(nameval, &name))
-            goto free_array_and_fail;
-
-        if (!gjs_object_require_property(context, props, "property list", name, &value))
+        if (!gjs_object_require_property(context, props, "property list", name, &value)) {
+            g_free(name);
             goto free_array_and_fail;
+        }
 
         switch (init_g_param_from_property(context, name,
                                            value,
                                            gtype,
                                            &gparam)) {
-        case SOME_ERROR_OCCURRED:
-            goto free_array_and_fail;
         case NO_SUCH_G_PROPERTY:
             gjs_throw(context, "No property %s on this GObject %s",
                          name, g_type_name(gtype));
+        case SOME_ERROR_OCCURRED:
+            g_free(name);
             goto free_array_and_fail;
         case VALUE_WAS_SET:
             break;
         }
 
+        g_free(name);
+
         g_array_append_val(gparams, gparam);
 
-        prop_id = JSVAL_VOID;
+        prop_id = JSID_VOID;
         if (!JS_NextProperty(context, iter, &prop_id))
             goto free_array_and_fail;
     }
@@ -560,13 +565,13 @@ wrapped_gobj_toggle_notify(gpointer      data,
 
     runtime = data;
 
-    /* The JSContext will be gone if runtime is being destroyed.
+    /* During teardown, this can return NULL if runtime is being destroyed.
      * In that case we effectively already converted to a weak ref without
      * doing anything since the keep alive will be collected.
      * Or if !is_last_ref, we do not want to convert to a strong
      * ref since we want everything collected on runtime destroy.
      */
-    context = gjs_runtime_peek_load_context(runtime);
+    context = gjs_runtime_get_current_context(runtime);
     if (!context)
         return;
 
@@ -598,7 +603,7 @@ wrapped_gobj_toggle_notify(gpointer      data,
          */
         if (priv->keep_alive == NULL) {
             gjs_debug_lifecycle(GJS_DEBUG_GOBJECT, "Adding object to keep alive");
-            priv->keep_alive = gjs_keep_alive_get_for_load_context(runtime);
+            priv->keep_alive = gjs_keep_alive_get_for_import_global(context);
             gjs_keep_alive_add_child(context, priv->keep_alive,
                                      gobj_no_longer_kept_alive_func,
                                      obj,
@@ -613,13 +618,9 @@ wrapped_gobj_toggle_notify(gpointer      data,
  * also, but can be replaced with another object to use instead as the
  * prototype.
  */
-static JSBool
-object_instance_constructor(JSContext *context,
-                            JSObject  *obj,
-                            uintN      argc,
-                            jsval     *argv,
-                            jsval     *retval)
+GJS_NATIVE_CONSTRUCTOR_DECLARE(object_instance)
 {
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(object_instance)
     ObjectInstance *priv;
     ObjectInstance *proto_priv;
     JSObject *proto;
@@ -627,29 +628,26 @@ object_instance_constructor(JSContext *context,
     JSClass *obj_class;
     JSClass *proto_class;
 
-    if (!gjs_check_constructing(context))
-        return JS_FALSE;
+    GJS_NATIVE_CONSTRUCTOR_PRELUDE(object_instance);
 
     priv = g_slice_new0(ObjectInstance);
 
     GJS_INC_COUNTER(object);
 
-    g_assert(priv_from_js(context, obj) == NULL);
-    JS_SetPrivate(context, obj, priv);
+    g_assert(priv_from_js(context, object) == NULL);
+    JS_SetPrivate(context, object, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_GOBJECT,
-                        "obj instance constructor, obj %p priv %p retval %p", obj, priv,
-                        JSVAL_IS_OBJECT(*retval) ?
-                        JSVAL_TO_OBJECT(*retval) : NULL);
+                        "obj instance constructor, obj %p priv %p", object, priv);
 
-    proto = JS_GetPrototype(context, obj);
+    proto = JS_GetPrototype(context, object);
     gjs_debug_lifecycle(GJS_DEBUG_GOBJECT, "obj instance __proto__ is %p", proto);
 
     /* If we're constructing the prototype, its __proto__ is not the same
      * class as us, but if we're constructing an instance, the prototype
      * has the same class.
      */
-    obj_class = JS_GET_CLASS(context, obj);
+    obj_class = JS_GET_CLASS(context, object);
     proto_class = JS_GET_CLASS(context, proto);
 
     is_proto = (obj_class != proto_class);
@@ -698,7 +696,7 @@ object_instance_constructor(JSContext *context,
                 return JS_FALSE;
             }
 
-            if (!object_instance_props_to_g_parameters(context, obj, argc, argv,
+            if (!object_instance_props_to_g_parameters(context, object, argc, argv,
                                                        gtype,
                                                        &params, &n_params)) {
                 return JS_FALSE;
@@ -729,10 +727,10 @@ object_instance_constructor(JSContext *context,
         }
 
         g_assert(peek_js_obj(context, priv->gobj) == NULL);
-        set_js_obj(context, priv->gobj, obj);
+        set_js_obj(context, priv->gobj, object);
 
 #if DEBUG_DISPOSE
-        g_object_weak_ref(priv->gobj, wrapped_gobj_dispose_notify, obj);
+        g_object_weak_ref(priv->gobj, wrapped_gobj_dispose_notify, object);
 #endif
 
         /* OK, here is where things get complicated. We want the
@@ -746,11 +744,11 @@ object_instance_constructor(JSContext *context,
          * the wrapper to be garbage collected (and thus unref the
          * wrappee).
          */
-        priv->keep_alive = gjs_keep_alive_get_for_load_context(JS_GetRuntime(context));
+        priv->keep_alive = gjs_keep_alive_get_for_import_global(context);
         gjs_keep_alive_add_child(context,
                                  priv->keep_alive,
                                  gobj_no_longer_kept_alive_func,
-                                 obj,
+                                 object,
                                  priv);
 
         g_object_add_toggle_ref(priv->gobj,
@@ -771,6 +769,10 @@ object_instance_constructor(JSContext *context,
                                     g_base_info_get_name ( (GIBaseInfo*) priv->info) ));
     }
 
+    GJS_NATIVE_CONSTRUCTOR_FINISH(object_instance);
+
+    gjs_maybe_gc(context);
+
     return JS_TRUE;
 }
 
@@ -794,7 +796,11 @@ object_instance_finalize(JSContext *context,
                                     g_base_info_get_name ( (GIBaseInfo*) priv->info) ));
 
     if (priv->gobj) {
-        g_assert(priv->gobj->ref_count > 0);
+        if (G_UNLIKELY (priv->gobj->ref_count <= 0)) {
+            g_error("Finalizing proxy for an already freed object of type: %s.%s\n",
+                    g_base_info_get_namespace((GIBaseInfo*) priv->info),
+                    g_base_info_get_name((GIBaseInfo*) priv->info));
+        }
         set_js_obj(context, priv->gobj, NULL);
         g_object_remove_toggle_ref(priv->gobj, wrapped_gobj_toggle_notify,
                                    JS_GetRuntime(context));
@@ -845,20 +851,20 @@ gjs_lookup_object_prototype(JSContext    *context,
 
 static JSBool
 real_connect_func(JSContext *context,
-                  JSObject  *obj,
                   uintN      argc,
-                  jsval     *argv,
-                  jsval     *retval,
+                  jsval     *vp,
                   gboolean  after)
 {
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
+    jsval *argv = JS_ARGV(context, vp);
     ObjectInstance *priv;
     GClosure *closure;
     gulong id;
     guint signal_id;
-    const char *signal_name;
+    char *signal_name;
     GQuark signal_detail;
-
-    *retval = INT_TO_JSVAL(0);
+    jsval retval;
+    JSBool ret = JS_FALSE;
 
     priv = priv_from_js(context, obj);
     gjs_debug_gsignal("connect obj %p priv %p argc %d", obj, priv, argc);
@@ -885,7 +891,7 @@ real_connect_func(JSContext *context,
         return JS_FALSE;
     }
 
-    signal_name = gjs_string_get_ascii_checked(context, argv[0]);
+    signal_name = gjs_string_get_ascii(context, argv[0]);
     if (signal_name == NULL) {
         return JS_FALSE;
     }
@@ -898,58 +904,57 @@ real_connect_func(JSContext *context,
         gjs_throw(context, "No signal '%s' on object '%s'",
                      signal_name,
                      g_type_name(G_OBJECT_TYPE(priv->gobj)));
-        return JS_FALSE;
+        goto out;
     }
 
     closure = gjs_closure_new_for_signal(context, JSVAL_TO_OBJECT(argv[1]), "signal callback", signal_id);
     if (closure == NULL)
-        return JS_FALSE;
+        goto out;
 
     id = g_signal_connect_closure(priv->gobj,
                                   signal_name,
                                   closure,
                                   after);
 
-    if (!JS_NewNumberValue(context, id, retval)) {
+    if (!JS_NewNumberValue(context, id, &retval)) {
         g_signal_handler_disconnect(priv->gobj, id);
-        return JS_FALSE;
+        goto out;
     }
+    
+    JS_SET_RVAL(context, vp, retval);
 
-    return JS_TRUE;
+    ret = JS_TRUE;
+ out:
+    g_free(signal_name);
+    return ret;
 }
 
 static JSBool
 connect_after_func(JSContext *context,
-                   JSObject  *obj,
                    uintN      argc,
-                   jsval     *argv,
-                   jsval     *retval)
+                   jsval     *vp)
 {
-    return real_connect_func(context, obj, argc, argv, retval, TRUE);
+    return real_connect_func(context, argc, vp, TRUE);
 }
 
 static JSBool
 connect_func(JSContext *context,
-             JSObject  *obj,
              uintN      argc,
-             jsval     *argv,
-             jsval     *retval)
+             jsval     *vp)
 {
-    return real_connect_func(context, obj, argc, argv, retval, FALSE);
+    return real_connect_func(context, argc, vp, FALSE);
 }
 
 static JSBool
 disconnect_func(JSContext *context,
-                JSObject  *obj,
                 uintN      argc,
-                jsval     *argv,
-                jsval     *retval)
+                jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     ObjectInstance *priv;
     gulong id;
 
-    *retval = JSVAL_VOID;
-
     priv = priv_from_js(context, obj);
     gjs_debug_gsignal("disconnect obj %p priv %p argc %d", obj, priv, argc);
 
@@ -973,28 +978,30 @@ disconnect_func(JSContext *context,
     id = JSVAL_TO_INT(argv[0]);
 
     g_signal_handler_disconnect(priv->gobj, id);
+    
+    JS_SET_RVAL(context, vp, JSVAL_VOID);
 
     return JS_TRUE;
 }
 
 static JSBool
 emit_func(JSContext *context,
-          JSObject  *obj,
           uintN      argc,
-          jsval     *argv,
-          jsval     *retval)
+          jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     ObjectInstance *priv;
     guint signal_id;
     GQuark signal_detail;
     GSignalQuery signal_query;
-    const char *signal_name;
+    char *signal_name;
     GValue *instance_and_args;
     GValue rvalue;
     unsigned int i;
     gboolean failed;
-
-    *retval = JSVAL_VOID;
+    jsval retval;
+    JSBool ret = JS_FALSE;
 
     priv = priv_from_js(context, obj);
     gjs_debug_gsignal("emit obj %p priv %p argc %d", obj, priv, argc);
@@ -1016,7 +1023,7 @@ emit_func(JSContext *context,
         return JS_FALSE;
     }
 
-    signal_name = gjs_string_get_ascii_checked(context,
+    signal_name = gjs_string_get_ascii(context,
                                                   argv[0]);
     if (signal_name == NULL)
         return JS_FALSE;
@@ -1029,7 +1036,7 @@ emit_func(JSContext *context,
         gjs_throw(context, "No signal '%s' on object '%s'",
                      signal_name,
                      g_type_name(G_OBJECT_TYPE(priv->gobj)));
-        return JS_FALSE;
+        goto out;
     }
 
     g_signal_query(signal_id, &signal_query);
@@ -1040,7 +1047,7 @@ emit_func(JSContext *context,
                      g_type_name(G_OBJECT_TYPE(priv->gobj)),
                      signal_query.n_params,
                      argc - 1);
-        return JS_FALSE;
+        goto out;
     }
 
     if (signal_query.return_type != G_TYPE_NONE) {
@@ -1075,7 +1082,7 @@ emit_func(JSContext *context,
 
     if (signal_query.return_type != G_TYPE_NONE) {
         if (!gjs_value_from_g_value(context,
-                                       retval,
+                                       &retval,
                                        &rvalue))
             failed = TRUE;
 
@@ -1086,7 +1093,13 @@ emit_func(JSContext *context,
         g_value_unset(&instance_and_args[i]);
     }
 
-    return !failed;
+    if (!failed)
+        JS_SET_RVAL(context, vp, retval);
+
+    ret = !failed;
+ out:
+    g_free(signal_name);
+    return ret;
 }
 
 /* Default spidermonkey toString is worthless.  Replace it
@@ -1095,18 +1108,16 @@ emit_func(JSContext *context,
  */
 static JSBool
 to_string_func(JSContext *context,
-               JSObject  *obj,
                uintN      argc,
-               jsval     *argv,
-               jsval     *retval)
+               jsval     *vp)
 {
+    JSObject *obj = JS_THIS_OBJECT(context, vp);
     ObjectInstance *priv;
     char *strval;
     JSBool ret;
     const char *namespace;
     const char *name;
-
-    *retval = JSVAL_VOID;
+    jsval retval;
 
     priv = priv_from_js(context, obj);
 
@@ -1122,7 +1133,9 @@ to_string_func(JSContext *context,
         strval = g_strdup_printf ("[object instance proxy GIName:%s.%s jsobj@%p native@%p]", namespace, name, obj, priv->gobj);
     }
 
-    ret = gjs_string_from_utf8 (context, strval, -1, retval);
+    ret = gjs_string_from_utf8 (context, strval, -1, &retval);
+    if (ret)
+        JS_SET_RVAL(context, vp, retval);
     g_free (strval);
     return ret;
 }
@@ -1158,11 +1171,11 @@ static JSPropertySpec gjs_object_instance_proto_props[] = {
 };
 
 static JSFunctionSpec gjs_object_instance_proto_funcs[] = {
-    { "connect", connect_func, 0, 0 },
-    { "connect_after", connect_after_func, 0, 0 },
-    { "disconnect", disconnect_func, 0, 0 },
-    { "emit", emit_func, 0, 0 },
-    { "toString", to_string_func, 0, 0 },
+    { "connect", (JSNative)connect_func, 0, JSFUN_FAST_NATIVE },
+    { "connect_after", (JSNative)connect_after_func, 0, JSFUN_FAST_NATIVE },
+    { "disconnect", (JSNative)disconnect_func, 0, JSFUN_FAST_NATIVE },
+    { "emit", (JSNative)emit_func, 0, JSFUN_FAST_NATIVE },
+    { "toString", (JSNative)to_string_func, 0, JSFUN_FAST_NATIVE },
     { NULL }
 };
 
@@ -1374,7 +1387,7 @@ gjs_define_object_class(JSContext     *context,
                                         * none - just name the prototype like
                                         * Math - rarely correct)
                                         */
-                                       object_instance_constructor,
+                                       gjs_object_instance_constructor,
                                        /* number of constructor args */
                                        0,
                                        /* props of prototype */
