@@ -60,7 +60,7 @@ resolve_namespace_object(JSContext  *context,
     jsval versions_val;
     JSObject *versions;
     jsval version_val;
-    const char *version;
+    char *version;
     JSObject *result;
 
     JS_BeginRequest(context);
@@ -78,7 +78,7 @@ resolve_namespace_object(JSContext  *context,
     version = NULL;
     if (JS_GetProperty(context, versions, ns_name, &version_val) &&
         JSVAL_IS_STRING(version_val)) {
-        version = gjs_string_get_ascii(version_val);
+        version = gjs_string_get_ascii(context, version_val);
     }
 
     repo = g_irepository_get_default();
@@ -90,10 +90,13 @@ resolve_namespace_object(JSContext  *context,
                   "Requiring %s, version %s: %s",
                   ns_name, version?version:"none", error->message);
         g_error_free(error);
+        g_free(version);
         JS_EndRequest(context);
         return NULL;
     }
 
+    g_free(version);
+
     /* Defines a property on "obj" (the javascript repo object)
      * with the given namespace name, pointing to that namespace
      * in the repo.
@@ -124,7 +127,8 @@ repo_new_resolve(JSContext *context,
                  JSObject **objp)
 {
     Repo *priv;
-    const char *name;
+    char *name;
+    JSBool ret = JS_TRUE;
 
     *objp = NULL;
 
@@ -134,23 +138,25 @@ repo_new_resolve(JSContext *context,
     /* let Object.prototype resolve these */
     if (strcmp(name, "valueOf") == 0 ||
         strcmp(name, "toString") == 0)
-        return JS_TRUE;
+        goto out;
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GREPO, "Resolve prop '%s' hook obj %p priv %p", name, obj, priv);
 
-    if (priv == NULL)
-        return JS_TRUE; /* we are the prototype, or have the wrong class */
+    if (priv == NULL) /* we are the prototype, or have the wrong class */
+        goto out;
 
     JS_BeginRequest(context);
     if (resolve_namespace_object(context, obj, name) == NULL) {
-        JS_EndRequest(context);
-        return JS_FALSE;
+        ret = JS_FALSE;
     } else {
         *objp = obj; /* store the object we defined the prop in */
-        JS_EndRequest(context);
-        return JS_TRUE;
     }
+    JS_EndRequest(context);
+
+ out:
+    g_free(name);
+    return ret;
 }
 
 /* If we set JSCLASS_CONSTRUCT_PROTOTYPE flag, then this is called on
@@ -161,24 +167,24 @@ repo_new_resolve(JSContext *context,
  * identify the prototype as an object of our class with NULL private
  * data.
  */
-static JSBool
-repo_constructor(JSContext *context,
-                 JSObject  *obj,
-                 uintN      argc,
-                 jsval     *argv,
-                 jsval     *retval)
+GJS_NATIVE_CONSTRUCTOR_DECLARE(repo)
 {
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(repo)
     Repo *priv;
 
+    GJS_NATIVE_CONSTRUCTOR_PRELUDE(repo);
+
     priv = g_slice_new0(Repo);
 
     GJS_INC_COUNTER(repo);
 
-    g_assert(priv_from_js(context, obj) == NULL);
-    JS_SetPrivate(context, obj, priv);
+    g_assert(priv_from_js(context, object) == NULL);
+    JS_SetPrivate(context, object, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_GREPO,
-                        "repo constructor, obj %p priv %p", obj, priv);
+                        "repo constructor, obj %p priv %p", object, priv);
+
+    GJS_NATIVE_CONSTRUCTOR_FINISH(repo);
 
     return JS_TRUE;
 }
@@ -216,7 +222,7 @@ static struct JSClass gjs_repo_class = {
     JS_PropertyStub,
     JS_PropertyStub,
     JS_PropertyStub,
-    JS_PropertyStub,
+    JS_StrictPropertyStub,
     JS_EnumerateStub,
     (JSResolveOp) repo_new_resolve, /* needs cast since it's the new resolve signature */
     JS_ConvertStub,
@@ -254,7 +260,7 @@ repo_new(JSContext *context)
                                   * none - just name the prototype like
                                   * Math - rarely correct)
                                   */
-                                 repo_constructor,
+                                 gjs_repo_constructor,
                                  /* number of constructor args */
                                  0,
                                  /* props of prototype */
