@@ -26,7 +26,8 @@
 
 #include "gjs-dbus/dbus.h"
 
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <gjs/compat.h>
 
 #include <util/log.h>
 
@@ -56,15 +57,8 @@ typedef struct {
      * around.  However, the alternatives are complicated, and
      * SpiderMonkey currently uses mark-and-sweep, so I think this
      * should be fine. We'll see I guess.
-     *
-     * Saving the context is also questionable: Objects can jump between
-     * contexts (they are only permanently bound to a runtime). However,
-     * we assume, AFAIK safely, that in our usage model the context where
-     * we created the exports object will always be the right one to
-     * invoke incoming calls. A context contains the global scope and an
-     * execution stack.
      */
-    JSContext  *context;
+    JSRuntime *runtime;
     JSObject   *object;
 
     DBusBusType which_bus;
@@ -156,7 +150,7 @@ dbus_reply_from_exception_and_sender(JSContext    *context,
 {
     char *s;
     jsval exc;
-    const char *name = NULL;
+    char *name = NULL;
     jsval nameval;
 
     *reply_p = NULL;
@@ -167,10 +161,12 @@ dbus_reply_from_exception_and_sender(JSContext    *context,
     if (JSVAL_IS_OBJECT(exc) &&
         gjs_object_get_property(context, JSVAL_TO_OBJECT(exc),
                                 "dbusErrorName", &nameval))
-        name = gjs_string_get_ascii_checked(context, nameval);
+        name = gjs_string_get_ascii(context, nameval);
 
-    if (!gjs_log_exception(context, &s))
+    if (!gjs_log_exception(context, &s)) {
+        g_free(name);
         return JS_FALSE;
+    }
 
     gjs_debug(GJS_DEBUG_DBUS,
               "JS exception we will send as dbus reply to %s: %s",
@@ -182,6 +178,7 @@ dbus_reply_from_exception_and_sender(JSContext    *context,
     dbus_message_set_reply_serial(*reply_p, serial);
     dbus_message_set_no_reply(*reply_p, TRUE);
     dbus_message_set_error_name(*reply_p, name ? name : DBUS_ERROR_FAILED);
+    g_free(name);
     if (s != NULL) {
         DBusMessageIter iter;
 
@@ -203,21 +200,21 @@ dbus_reply_from_exception_and_sender(JSContext    *context,
 static JSBool
 signature_from_method(JSContext   *context,
                       JSObject    *method_obj,
-                      const char **signature)
+                      char       **signature)
 {
     jsval signature_value;
 
     if (gjs_object_get_property(context,
                                 method_obj, "outSignature",
                                 &signature_value)) {
-        *signature = gjs_string_get_ascii_checked(context,
+        *signature = gjs_string_get_ascii(context,
                                                   signature_value);
         if (*signature == NULL) {
             return JS_FALSE;
         }
     } else {
         /* We default to a{sv} */
-        *signature = "a{sv}";
+        *signature = g_strdup("a{sv}");
     }
 
     return JS_TRUE;
@@ -298,7 +295,7 @@ invoke_js_from_dbus(JSContext   *context,
     jsval rval;
     DBusMessageIter arg_iter;
     GjsRootedArray *values;
-    const char *signature;
+    char *signature;
 
     if (JS_IsExceptionPending(context)) {
         gjs_debug(GJS_DEBUG_DBUS,
@@ -321,7 +318,7 @@ invoke_js_from_dbus(JSContext   *context,
     argv = gjs_rooted_array_get_data(context, values);
 
     rval = JSVAL_VOID;
-    JS_AddRoot(context, &rval);
+    JS_AddValueRoot(context, &rval);
 
     gjs_js_push_current_message(method_call);
     if (!gjs_call_function_value(context,
@@ -334,8 +331,6 @@ invoke_js_from_dbus(JSContext   *context,
         gjs_debug(GJS_DEBUG_DBUS,
                   "dbus method invocation failed");
 
-        JS_RemoveRoot(context, &rval);
-
         if (!dbus_reply_from_exception(context, method_call, &reply))
             gjs_debug(GJS_DEBUG_DBUS,
                       "dbus method invocation failed but no exception was set?");
@@ -365,9 +360,11 @@ invoke_js_from_dbus(JSContext   *context,
                                    dbus_message_get_serial(method_call),
                                    rval);
 
+    g_free(signature);
+
  out:
     gjs_rooted_array_free(context, values, TRUE);
-    JS_RemoveRoot(context, &rval);
+    JS_RemoveValueRoot(context, &rval);
 
     gjs_js_pop_current_message();
 
@@ -386,23 +383,21 @@ invoke_js_from_dbus(JSContext   *context,
 
 static JSBool
 async_call_callback(JSContext *context,
-                    JSObject  *obj,
                     uintN      argc,
-                    jsval     *argv,
-                    jsval     *retval)
+                    jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     DBusConnection *connection;
     DBusBusType which_bus;
     DBusMessage *reply;
     JSObject *callback_object;
-    const char *sender;
+    char *sender;
     dbus_uint32_t serial;
     jsval prop_value;
-    const char *signature;
+    char *signature = NULL;
     gboolean thrown;
 
-    *retval = JSVAL_VOID;
-    callback_object = JSVAL_TO_OBJECT(JS_ARGV_CALLEE(argv));
+    callback_object = JSVAL_TO_OBJECT(JS_CALLEE(context, vp));
     reply = NULL;
     thrown = FALSE;
 
@@ -416,7 +411,7 @@ async_call_callback(JSContext *context,
         gjs_log_and_keep_exception(context, NULL);
         return JS_FALSE;
     }
-    sender = gjs_string_get_ascii_checked(context, prop_value);
+    sender = gjs_string_get_ascii(context, prop_value);
     if (!sender)
         return JS_FALSE;
 
@@ -426,10 +421,10 @@ async_call_callback(JSContext *context,
                                      "_dbusSerial",
                                      &prop_value)) {
         gjs_log_and_keep_exception(context, NULL);
-        return JS_FALSE;
+        goto fail;
     }
     if (!JS_ValueToECMAUint32(context, prop_value, &serial))
-        return JS_FALSE;
+        goto fail;
 
     if (!gjs_object_require_property(context,
                                      callback_object,
@@ -437,7 +432,7 @@ async_call_callback(JSContext *context,
                                      "_dbusBusType",
                                      &prop_value)) {
         gjs_log_and_keep_exception(context, NULL);
-        return JS_FALSE;
+        goto fail;
     }
     which_bus = JSVAL_TO_INT(prop_value);
 
@@ -452,11 +447,12 @@ async_call_callback(JSContext *context,
         thrown = TRUE;
         goto out;
     }
-    signature = gjs_string_get_ascii_checked(context, prop_value);
+    signature = gjs_string_get_ascii(context, prop_value);
     if (!signature)
-        return JS_FALSE;
+        goto fail;
 
-    if (argc != 1) {
+    if ((argc == 0 && !g_str_equal(signature, ""))
+        || argc > 1) {
         gjs_throw(context, "The callback to async DBus calls takes one argument, "
                   "the return value or array of return values");
         thrown = TRUE;
@@ -476,6 +472,9 @@ async_call_callback(JSContext *context,
                       "dbus method invocation failed but no exception was set?");
     }
 
+    g_free(sender);
+    g_free(signature);
+
     if (reply) {
         gjs_dbus_add_bus_weakref(which_bus, &connection);
         if (!connection) {
@@ -489,8 +488,14 @@ async_call_callback(JSContext *context,
         gjs_dbus_remove_bus_weakref(which_bus, &connection);
         dbus_message_unref(reply);
     }
+    if (!thrown)
+        JS_SET_RVAL(context, vp, JSVAL_VOID);
 
     return (thrown == FALSE);
+
+ fail:
+    g_free(sender);
+    return JS_FALSE;
 }
 
 /* returns an error message or NULL */
@@ -512,7 +517,7 @@ invoke_js_async_from_dbus(JSContext   *context,
     jsval serial_value;
     gboolean thrown;
     jsval ignored;
-    const char *signature;
+    char *signature;
     JSString *signature_string;
 
     reply = NULL;
@@ -537,8 +542,8 @@ invoke_js_async_from_dbus(JSContext   *context,
 
     /* we will add an argument, the callback */
     callback = JS_NewFunction(context,
-                              async_call_callback,
-                              1, 0,
+                              (JSNative)async_call_callback,
+                              1, JSFUN_FAST_NATIVE,
                               NULL,
                               "" /* anonymous */);
 
@@ -606,6 +611,7 @@ invoke_js_async_from_dbus(JSContext   *context,
     }
 
     signature_string = JS_NewStringCopyZ(context, signature);
+    g_free(signature);
     if (!signature_string) {
         thrown = TRUE;
         goto out;
@@ -827,9 +833,9 @@ unpack_property_details(JSContext       *context,
     jsval name_val;
     jsval signature_val;
     jsval access_val;
-    const char *name;
-    const char *signature;
-    const char *access;
+    char *name = NULL;
+    char *signature = NULL;
+    char *access = NULL;
 
     if (!gjs_object_get_property(context,
                                  prop_description,
@@ -840,7 +846,7 @@ unpack_property_details(JSContext       *context,
         return JS_FALSE;
     }
 
-    name = gjs_string_get_ascii_checked(context,
+    name = gjs_string_get_ascii(context,
                                         name_val);
     if (name == NULL) {
         return JS_FALSE;
@@ -853,14 +859,13 @@ unpack_property_details(JSContext       *context,
         gjs_throw(context,
                   "Property %s has no signature",
                   name);
-        return JS_FALSE;
+        goto fail;
     }
 
-    signature = gjs_string_get_ascii_checked(context,
+    signature = gjs_string_get_ascii(context,
                                              signature_val);
-    if (signature == NULL) {
-        return JS_FALSE;
-    }
+    if (signature == NULL)
+        goto fail;
 
     if (!gjs_object_get_property(context,
                                  prop_description,
@@ -869,14 +874,13 @@ unpack_property_details(JSContext       *context,
         gjs_throw(context,
                   "Property %s has no access",
                   name);
-        return JS_FALSE;
+        goto fail;
     }
 
-    access = gjs_string_get_ascii_checked(context,
+    access = gjs_string_get_ascii(context,
                                           access_val);
-    if (access == NULL) {
-        return JS_FALSE;
-    }
+    if (access == NULL)
+        goto fail;
 
     g_assert(name && signature && access);
 
@@ -889,13 +893,20 @@ unpack_property_details(JSContext       *context,
         details->writable = TRUE;
     } else {
         gjs_throw(context, "Unknown access on property, should be readwrite read or write");
-        return JS_FALSE;
+        goto fail;
     }
 
-    details->name = g_strdup(name);
-    details->signature = g_strdup(signature);
+    details->name = name;
+    details->signature = signature;
 
+    g_free(access);
     return JS_TRUE;
+
+ fail:
+    g_free(access);
+    g_free(signature);
+    g_free(name);
+    return JS_FALSE;
 }
 
 /* FALSE on exception, NULL property name in details if no such
@@ -1056,12 +1067,12 @@ handle_get_property(JSContext      *context,
     }
 
     value = JSVAL_VOID;
-    JS_AddRoot(context, &value);
+    JS_AddValueRoot(context, &value);
     if (!gjs_object_require_property(context, obj,
                                      "DBus GetProperty callee",
                                      prop_name, &value)) {
 
-        JS_RemoveRoot(context, &value);
+        JS_RemoveValueRoot(context, &value);
         property_details_clear(&details);
 
         dbus_reply_from_exception(context, message,
@@ -1084,7 +1095,7 @@ handle_get_property(JSContext      *context,
                                   &variant_iter, &sig_iter)) {
 
         property_details_clear(&details);
-        JS_RemoveRoot(context, &value);
+        JS_RemoveValueRoot(context, &value);
 
         dbus_message_unref(reply);
         reply = NULL;
@@ -1096,7 +1107,7 @@ handle_get_property(JSContext      *context,
 
     dbus_message_iter_close_container(&iter, &variant_iter);
 
-    JS_RemoveRoot(context, &value);
+    JS_RemoveValueRoot(context, &value);
 
     property_details_clear(&details);
 
@@ -1175,13 +1186,13 @@ handle_get_all_properties(JSContext      *context,
             }
 
             value = JSVAL_VOID;
-            JS_AddRoot(context, &value);
+            JS_AddValueRoot(context, &value);
             if (!gjs_object_require_property(context, obj,
                                              "DBus GetAllProperties callee",
                                              details.name, &value)) {
 
                 property_details_clear(&details);
-                JS_RemoveRoot(context, &value);
+                JS_RemoveValueRoot(context, &value);
 
                 goto js_exception;
             }
@@ -1202,13 +1213,13 @@ handle_get_all_properties(JSContext      *context,
 #ifdef HAVE_DBUS_MESSAGE_ITER_ABANDON_CONTAINER
                 dbus_message_iter_abandon_container(&entry_iter, &entry_value_iter);
 #endif
-                JS_RemoveRoot(context, &value);
+                JS_RemoveValueRoot(context, &value);
                 property_details_clear(&details);
                 goto js_exception;
             }
             dbus_message_iter_close_container(&entry_iter, &entry_value_iter);
 
-            JS_RemoveRoot(context, &value);
+            JS_RemoveValueRoot(context, &value);
 
             dbus_message_iter_close_container(&dict_iter, &entry_iter);
             property_details_clear(&details);
@@ -1310,18 +1321,18 @@ handle_set_property(JSContext      *context,
     property_details_clear(&details);
 
     value = JSVAL_VOID;
-    JS_AddRoot(context, &value);
+    JS_AddValueRoot(context, &value);
     gjs_js_one_value_from_dbus(context, &iter, &value);
 
     if (dbus_reply_from_exception(context, message, &reply)) {
-        JS_RemoveRoot(context, &value);
+        JS_RemoveValueRoot(context, &value);
         return reply;
     }
 
     /* this throws on oom or if prop is read-only for example */
     JS_SetProperty(context, obj, prop_name, &value);
 
-    JS_RemoveRoot(context, &value);
+    JS_RemoveValueRoot(context, &value);
 
     if (!dbus_reply_from_exception(context, message, &reply)) {
         g_assert(reply == NULL);
@@ -1403,18 +1414,18 @@ handle_introspect(JSContext      *context,
                                children[i]);
     }
 
-    JS_AddRoot(context, &props_iter);
-    JS_AddRoot(context, &key_str);
+    JS_AddObjectRoot(context, &props_iter);
+    JS_AddStringRoot(context, &key_str);
     props_iter = JS_NewPropertyIterator(context, dir_obj);
 
-    prop_id = JSVAL_VOID;
+    prop_id = JSID_VOID;
     if (!JS_NextProperty(context, props_iter, &prop_id)) {
         gjs_debug(GJS_DEBUG_DBUS,
                   "Failed to get next property iterating dbus directory");
         goto out;
     }
 
-    while (prop_id != JSVAL_VOID) {
+    while (!JSID_IS_VOID(prop_id)) {
         char *key;
         jsval keyval;
         jsval valueval = JSVAL_VOID;
@@ -1438,13 +1449,15 @@ handle_introspect(JSContext      *context,
             goto out;
         }
 
-        key = JS_GetStringBytes(key_str);
+        if (!gjs_string_to_utf8(context, keyval, &key))
+            goto out;
 
         if (!gjs_object_require_property(context, dir_obj,
                                          "dbus directory",
                                          key, &valueval)) {
             gjs_debug(GJS_DEBUG_DBUS,
                       "Somehow failed to get property of dbus object");
+            g_free(key);
             goto out;
         }
 
@@ -1453,8 +1466,9 @@ handle_introspect(JSContext      *context,
             g_string_append_printf(doc, "  <node name=\"%s\"/>\n",
                                    key);
         }
+        g_free(key);
 
-        prop_id = JSVAL_VOID;
+        prop_id = JSID_VOID;
         if (!JS_NextProperty(context, props_iter, &prop_id)) {
             gjs_debug(GJS_DEBUG_DBUS,
                       "Failed to get next property iterating dbus object");
@@ -1493,8 +1507,8 @@ handle_introspect(JSContext      *context,
     dbus_connection_send(connection, reply, NULL);
 
  out:
-    JS_RemoveRoot(context, &key_str);
-    JS_RemoveRoot(context, &props_iter);
+    JS_RemoveStringRoot(context, &key_str);
+    JS_RemoveObjectRoot(context, &props_iter);
 
     if (reply != NULL)
         dbus_message_unref(reply);
@@ -1515,6 +1529,7 @@ on_message(DBusConnection *connection,
 {
     const char *path;
     DBusHandlerResult result;
+    JSContext *context;
     JSObject *obj, *dir_obj = NULL;
     const char *method_name;
     char *async_method_name;
@@ -1529,15 +1544,17 @@ on_message(DBusConnection *connection,
     if (dbus_message_get_type(message) != DBUS_MESSAGE_TYPE_METHOD_CALL)
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
-    JS_BeginRequest(priv->context);
+    context = gjs_runtime_get_current_context(priv->runtime);
+
+    JS_BeginRequest(context);
     method_value = JSVAL_VOID;
-    JS_AddRoot(priv->context, &method_value);
+    JS_AddValueRoot(context, &method_value);
 
     result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
     path = dbus_message_get_path(message);
 
-    obj = find_js_property_by_path(priv->context,
+    obj = find_js_property_by_path(context,
                                    priv->object,
                                    path, &dir_obj);
 
@@ -1556,7 +1573,7 @@ on_message(DBusConnection *connection,
                   path);
 
         if (dir_obj != NULL)
-            result = handle_introspect(priv->context,
+            result = handle_introspect(context,
                                        connection,
                                        dir_obj, obj,
                                        message);
@@ -1581,7 +1598,7 @@ on_message(DBusConnection *connection,
                   "Properties request %s on %s",
                   method_name,
                   iface ? iface : "MISSING INTERFACE");
-        result = handle_properties(priv->context, connection,
+        result = handle_properties(context, connection,
                                    obj, message, method_name);
         goto out;
     }
@@ -1589,7 +1606,7 @@ on_message(DBusConnection *connection,
     async_method_name = g_strdup_printf("%sAsync", method_name);
 
     /* try first if an async version exists */
-    if (find_method(priv->context,
+    if (find_method(context,
                     obj,
                     async_method_name,
                     &method_value)) {
@@ -1598,7 +1615,7 @@ on_message(DBusConnection *connection,
                   "Invoking async method %s on JS obj at dbus path %s",
                   async_method_name, path);
 
-        reply = invoke_js_async_from_dbus(priv->context,
+        reply = invoke_js_async_from_dbus(context,
                                           priv->which_bus,
                                           message,
                                           obj,
@@ -1607,7 +1624,7 @@ on_message(DBusConnection *connection,
         result = DBUS_HANDLER_RESULT_HANDLED;
 
         /* otherwise try the sync version */
-    } else if (find_method(priv->context,
+    } else if (find_method(context,
                            obj,
                            method_name,
                            &method_value)) {
@@ -1616,7 +1633,7 @@ on_message(DBusConnection *connection,
                   "Invoking method %s on JS obj at dbus path %s",
                   method_name, path);
 
-        reply = invoke_js_from_dbus(priv->context,
+        reply = invoke_js_from_dbus(context,
                                     message,
                                     obj,
                                     JSVAL_TO_OBJECT(method_value));
@@ -1638,8 +1655,8 @@ on_message(DBusConnection *connection,
  out:
     if (async_method_name)
         g_free(async_method_name);
-    JS_RemoveRoot(priv->context, &method_value);
-    JS_EndRequest(priv->context);
+    JS_RemoveValueRoot(context, &method_value);
+    JS_EndRequest(context);
     return result;
 }
 
@@ -1659,20 +1676,21 @@ on_message(DBusConnection *connection,
 static JSBool
 exports_new_resolve(JSContext *context,
                     JSObject  *obj,
-                    jsval      id,
+                    jsid       id,
                     uintN      flags,
                     JSObject **objp)
 {
     Exports *priv;
-    const char *name;
+    char *name;
 
     *objp = NULL;
 
-    if (!gjs_get_string_id(id, &name))
+    if (!gjs_get_string_id(context, id, &name))
         return JS_TRUE; /* not resolved, but no error */
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_DBUS, "Resolve prop '%s' hook obj %p priv %p", name, obj, priv);
+    g_free(name);
 
     if (priv == NULL)
         return JS_TRUE; /* we are the prototype, or have the wrong class */
@@ -1688,27 +1706,27 @@ exports_new_resolve(JSContext *context,
  * identify the prototype as an object of our class with NULL private
  * data.
  */
-static JSBool
-exports_constructor(JSContext *context,
-                    JSObject  *obj,
-                    uintN      argc,
-                    jsval     *argv,
-                    jsval     *retval)
+GJS_NATIVE_CONSTRUCTOR_DECLARE(js_exports)
 {
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(js_exports)
     Exports *priv;
 
+    GJS_NATIVE_CONSTRUCTOR_PRELUDE(js_exports);
+
     priv = g_slice_new0(Exports);
 
     GJS_INC_COUNTER(dbus_exports);
 
-    g_assert(priv_from_js(context, obj) == NULL);
-    JS_SetPrivate(context, obj, priv);
+    g_assert(priv_from_js(context, object) == NULL);
+    JS_SetPrivate(context, object, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_DBUS,
-                        "exports constructor, obj %p priv %p", obj, priv);
+                        "exports constructor, obj %p priv %p", object, priv);
 
-    priv->context = context;
-    priv->object = obj;
+    priv->runtime = JS_GetRuntime(context);
+    priv->object = object;
+
+    GJS_NATIVE_CONSTRUCTOR_FINISH(js_exports);
 
     return JS_TRUE;
 }
@@ -1785,7 +1803,7 @@ static struct JSClass gjs_js_exports_class = {
     JS_PropertyStub,
     JS_PropertyStub,
     JS_PropertyStub,
-    JS_PropertyStub,
+    JS_StrictPropertyStub,
     JS_EnumerateStub,
     (JSResolveOp) exports_new_resolve, /* needs cast since it's the new resolve signature */
     JS_ConvertStub,
@@ -1812,7 +1830,7 @@ exports_new(JSContext  *context,
     JSObject *global;
 
     /* put constructor for DBusExports() in the global namespace */
-    global = JS_GetGlobalObject(context);
+    global = gjs_get_import_global(context);
 
     if (!gjs_object_has_property(context, global, gjs_js_exports_class.name)) {
         JSObject *prototype;
@@ -1828,7 +1846,7 @@ exports_new(JSContext  *context,
                                   * none - just name the prototype like
                                   * Math - rarely correct)
                                   */
-                                 exports_constructor,
+                                 gjs_js_exports_constructor,
                                  /* number of constructor args */
                                  0,
                                  /* props of prototype */
@@ -1848,7 +1866,7 @@ exports_new(JSContext  *context,
                   gjs_js_exports_class.name, prototype);
     }
 
-    exports = JS_ConstructObject(context, &gjs_js_exports_class, NULL, NULL);
+    exports = JS_ConstructObject(context, &gjs_js_exports_class, NULL, global);
     /* may be NULL */
 
     return exports;
@@ -1860,16 +1878,14 @@ gjs_js_define_dbus_exports(JSContext      *context,
                            DBusBusType     which_bus)
 {
     JSObject *exports;
-    JSContext *load_context;
     JSBool success;
 
     success = JS_FALSE;
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
+    JS_BeginRequest(context);
 
-    exports = exports_new(load_context, which_bus);
+    exports = exports_new(context, which_bus);
     if (exports == NULL) {
-        gjs_move_exception(load_context, context);
+        gjs_move_exception(context, context);
         goto fail;
     }
 
@@ -1885,6 +1901,6 @@ gjs_js_define_dbus_exports(JSContext      *context,
 
     success = JS_TRUE;
  fail:
-    JS_EndRequest(load_context);
+    JS_EndRequest(context);
     return success;
 }
